@@ -31,7 +31,7 @@ It is also the entry point to the Dpkg module hierarchy.
 use strict;
 use warnings;
 
-our $VERSION = '1.03';
+our $VERSION = '2.00';
 our @EXPORT_OK = qw(
     $PROGNAME
     $PROGVERSION
@@ -43,13 +43,6 @@ our @EXPORT_OK = qw(
     $LIBDIR
     $DATADIR
 );
-our @EXPORT = qw(
-    $version
-    $progname
-    $admindir
-    $dpkglibdir
-    $pkgdatadir
-);
 
 use Exporter qw(import);
 
@@ -101,7 +94,7 @@ Contains the path to the dpkg architecture tables directory.
 our ($PROGNAME) = $0 =~ m{(?:.*/)?([^/]*)};
 
 # The following lines are automatically fixed at install time
-our $PROGVERSION = '1.19.x';
+our $PROGVERSION = '1.20.x';
 our $PROGMAKE = $ENV{DPKG_PROGMAKE} // 'make';
 our $PROGTAR = $ENV{DPKG_PROGTAR} // 'tar';
 our $PROGPATCH = $ENV{DPKG_PROGPATCH} // 'patch';
@@ -113,14 +106,6 @@ our $DATADIR = '../data';
 
 $DATADIR = $ENV{DPKG_DATADIR} if defined $ENV{DPKG_DATADIR};
 
-# XXX: Backwards compatibility, to be removed on VERSION 2.00.
-## no critic (Variables::ProhibitPackageVars)
-our $version = $PROGVERSION;
-our $admindir = $ADMINDIR;
-our $dpkglibdir = $LIBDIR;
-our $pkgdatadir = $DATADIR;
-## use critic
-
 =head1 MODULES
 
 The following is the list of public modules within the Dpkg hierarchy. Only
@@ -266,6 +251,10 @@ Base object serializer.
 
 Common path handling functions.
 
+=item L<Dpkg::Source::Format>
+
+Parse and manipulate debian/source/format files.
+
 =item L<Dpkg::Source::Package>
 
 Extract Debian source packages.
@@ -286,6 +275,10 @@ Parse and manipulate Debian package versions.
 
 =head1 CHANGES
 
+=head2 Version 2.00 (dpkg 1.20.0)
+
+Remove variables: $version, $progname, $admindir, $dpkglibdir and $pkgdatadir.
+
 =head2 Version 1.03 (dpkg 1.18.24)
 
 New variable: $PROGPATCH.
