@@ -401,6 +401,7 @@ sub _generate_patch {
     my ($tarfile, %addonfile);
     my $comp_ext_regex = compression_get_file_extension_regex();
     my @origtarfiles;
+    my @origtarsigns;
     foreach my $file (sort $self->find_original_tarballs()) {
         if ($file =~ /\.orig\.tar\.$comp_ext_regex$/) {
             if (defined($tarfile)) {
@@ -408,26 +409,23 @@ sub _generate_patch {
                          'one is allowed'), $tarfile, $file);
             }
             $tarfile = $file;
-            push @origtarfiles, $file;
-            $self->add_file($file);
-            if (-e "$file.sig" and not -e "$file.asc") {
-                openpgp_sig_to_asc("$file.sig", "$file.asc");
-            }
-            if (-e "$file.asc") {
-                push @origtarfiles, "$file.asc";
-                $self->add_file("$file.asc")
-            }
         } elsif ($file =~ /\.orig-([[:alnum:]-]+)\.tar\.$comp_ext_regex$/) {
             $addonfile{$1} = $file;
-            push @origtarfiles, $file;
-            $self->add_file($file);
-            if (-e "$file.sig" and not -e "$file.asc") {
-                openpgp_sig_to_asc("$file.sig", "$file.asc");
-            }
-            if (-e "$file.asc") {
-                push @origtarfiles, "$file.asc";
-                $self->add_file("$file.asc");
-            }
+        } else {
+            next;
+        }
+
+        push @origtarfiles, $file;
+        $self->add_file($file);
+
+        # Check for an upstream signature.
+        if (-e "$file.sig" and not -e "$file.asc") {
+            openpgp_sig_to_asc("$file.sig", "$file.asc");
+        }
+        if (-e "$file.asc") {
+            push @origtarfiles, "$file.asc";
+            push @origtarsigns, "$file.asc";
+            $self->add_file("$file.asc")
         }
     }
 
@@ -439,6 +437,10 @@ sub _generate_patch {
             info(g_('building %s using existing %s'),
                  $self->{fields}{'Source'}, $origtarfile);
         }
+
+        if (@origtarsigns) {
+            $self->check_original_tarball_signature($dir, @origtarsigns);
+        }
     }
 
     # Unpack a second copy for comparison
@@ -587,17 +589,25 @@ sub do_build {
 
 sub _get_patch_header {
     my ($self, $dir) = @_;
+
     my $ph = File::Spec->catfile($dir, 'debian', 'source', 'local-patch-header');
     unless (-f $ph) {
         $ph = File::Spec->catfile($dir, 'debian', 'source', 'patch-header');
     }
-    my $text;
     if (-f $ph) {
-        open(my $ph_fh, '<', $ph) or syserr(g_('cannot read %s'), $ph);
-        $text = file_slurp($ph_fh);
-        close($ph_fh);
-        return $text;
+        return file_slurp($ph);
     }
+
+    if ($self->{options}->{single_debian_patch}) {
+        return <<'AUTOGEN_HEADER';
+This is an autogenerated patch header for a single-debian-patch file. The
+delta against upstream is either kept as a single patch, or maintained
+in some VCS, and exported as a single patch instead of more manageable
+atomic patches.
+
+AUTOGEN_HEADER
+    }
+
     my $ch_info = changelog_parse(offset => 0, count => 1,
         file => File::Spec->catfile($dir, 'debian', 'changelog'));
     return '' if not defined $ch_info;
@@ -613,6 +623,7 @@ it.\n";
     $header->{'Author'} = $ch_info->{'Maintainer'};
     my $yyyy_mm_dd = POSIX::strftime('%Y-%m-%d', gmtime);
 
+    my $text;
     $text = "$header";
     run_vendor_hook('extend-patch-header', \$text, $ch_info);
     $text .= "\n---
